@@ -15,9 +15,11 @@ from rcdesign.is456.rebar import (
     RebarGroup,
     ShearRebarGroup,
     LateralTie,
+    ShearRebarType,
     StressType,
     StressLabel,
 )
+from rcdesign.is456.design import Beam
 from rcdesign.utils import rootsearch, underline, header
 
 
@@ -128,22 +130,21 @@ class RectBeamSection:
         self.calc_stress_type(xu)
         k = xu / self.D
         ecy = self.csb.ecy
-        s = f"RECTANGULAR BEAM SECTION: {self.b} x {self.D}\n"
-        s += (
-            f"FLEXURE\nEquilibrium NA = {xu:.2f} (k = {k:.2f}) (ec_max = {ecmax:.6f})\n"
-        )
+        hdr0 = f"RECTANGULAR BEAM SECTION: {self.b} x {self.D}"
+        s = f"{header(hdr0, '~')}\n"
+        s += f"{header('FLEXURE', '=')}\nEquilibrium NA = {xu:.2f} (k = {k:.2f}) (ec_max = {ecmax:.6f})\n\n"
         fcc = self.csb._fc_(ecmax) * self.conc.fd
         Fc = self.b * self.csb.C(0, k, k, ecmax) * self.conc.fd * self.D
         Mc = self.csb.M(0, k, k) * self.conc.fd * self.b * self.D ** 2
         hdr1 = f"{'fck':>6} {' ':>8} {' ':>12} {'ec_max':>12} {'Type':>4} "
-        hdr1 += f"{' ':>8} {'f_cc':>6} {'C (kN)':>8} {'M (kNm)':>8}"
+        hdr1 += f"{' ':>8} {'f_c':>6} {'F (kN)':>8} {'M (kNm)':>8}"
         s += hdr1 + "\n" + underline(hdr1) + "\n"
         s += f"{self.conc.fck:6.2f} {' ':>8} {' ':>12} {ecmax:12.8f} {'C':>4} {' ':>8} {fcc:6.2f} "
         s += f"{Fc / 1e3:8.2f} {Mc/ 1e6:8.2f}\n{underline(hdr1)}\n\n"
         Ft = 0.0
         Mt = 0.0
-        hdr2 = f"{'fy':>6} {'Bars':>12} {'xc':>8} {'Strain':>12} {'Type':>4} {'f_sc':>8} {'f_cc':>6}"
-        hdr2 += f" {'C (kN)':>8} {'M (kNm)':>8}"
+        hdr2 = f"{'fy':>6} {'Bars':>12} {'xc':>8} {'Strain':>12} {'Type':>4} {'f_s':>8} {'f_c':>6}"
+        hdr2 += f" {'F (kN)':>8} {'M (kNm)':>8}"
         s += f"{hdr2}\n{underline(hdr2)}\n"
         for L in sorted(self.long_steel.layers):
             z = k - (L._xc / self.D)
@@ -171,11 +172,37 @@ class RectBeamSection:
         F = 0.0 if isclose(Fc + Ft, 0, abs_tol=1e-10) else Fc + Ft
         C_M = f"{F/1e3:8.2f} {(Mc + Mt)/1e6:8.2f}"
         s += f"{' ':>62} {C_M}\n"
-        s += f"SHEAR\n{self.shear_steel}\n"
-        s += f"CAPACITY\n{'Mu = ':>5}{self.Mu(xu, ecmax)/1e6:.2f} kNm\n"
-        vuc, vus = self.Vu(xu)
-        vu = vuc + sum(vus)
-        s += f"{'Vu = ':>5}{vu/1e3:.2f} kN\n"
+        s += f"{header('SHEAR', '=')}\n"
+        tauc = self.conc.tauc(self.pt(xu))
+        area = self.b * self.eff_d(xu)
+        vuc = area * tauc
+        hdr3 = f"{'Type':>14} {' ':>14} {'tau_c':>6} {'Area (mm^2)':>16} {' ':>8} {' ':>8} {'V_uc (kN)':>8}"
+        s += f"{header(hdr3)}\n"
+        s += f"{'Concrete':>14} {' ':>14} {tauc:6.2f} {area:16.2f} {' ':>8} {' ':>8} {vuc/1e3:8.2f}\n"
+        s += f"{underline(hdr3)}\n"
+        hdr4 = f"{'Type':>14} {'Variant':>14} {'f_y':>6} {'Bars':>16} {'s_v':>8} {'A_sv':>8} {'V_us (kN)':>8}"
+        s += f"{header(hdr4)}\n"
+        vus = 0.0
+        for sh_rein in self.shear_steel.shear_reinforcement:
+            data = sh_rein.report(self.eff_d(xu))
+            s += f"{data['label']:>14} {data['type']:>14} {data['fy']:6} "
+            if data["sh_type"] in [
+                ShearRebarType.SHEAR_REBAR_VERTICAL_STIRRUP,
+                ShearRebarType.SHEAR_REBAR_INCLINED_STIRRUP,
+            ]:
+                bar_info = f"{data['legs']}-{data['bar_dia']}#"
+            else:
+                bar_info = f"{data['bars']}"
+            s += f"{bar_info:>16} {data['sv']:8.1f} {data['Asv']:8.2f} {data['Vus']/1e3:8.2f}\n"
+            vus += data["Vus"]
+        vu = f"{(vuc + vus)/1e3:8.2f}"
+        s += f"{' ':>71} {underline(vu, '=')}\n{' ':>71} {vu}\n"
+        s += (
+            f"{header('CAPACITY', '=')}\n{'Mu = ':>5}{self.Mu(xu, ecmax)/1e6:.2f} kNm\n"
+        )
+        Vuc, Vus = self.Vu(xu)
+        Vu = Vuc + sum(Vus)
+        s += f"{'Vu = ':>5}{Vu/1e3:.2f} kN\n"
         return s
 
     def eff_d(self, xu: float) -> float:
@@ -206,6 +233,33 @@ class RectBeamSection:
         Mu = self.Mu(xu, ecmax)
         return xu, Mu
 
+    def design_singly(self, bar_dia: float, Mu: float) -> Tuple[float, float]:
+        beam = Beam()
+        fck = self.conc.fck
+        bottom_layer = self.long_steel.layers[-1]
+        fy = bottom_layer.rebar.fy
+        fd = bottom_layer.rebar.fd
+        d = self.D - self.clear_cover - bar_dia / 2
+        dc = self.clear_cover + bar_dia / 2
+        Mulim = beam.Mulim_const(fy) * fck * self.b * d ** 2
+        if Mu < Mulim:
+            s = "Singly reinforced"
+            ast = beam.reqd_Ast(fck, fy, self.b, d, Mu)
+            asc = 0.0
+        else:
+            s = "Doubly reinforced"
+            ast1 = beam.reqd_Ast(fck, fy, self.b, d, Mulim)
+            Mu2 = Mu - Mulim
+            ast2 = Mu2 / (fd * (d - dc))
+            ast = ast1 + ast2
+            xu = beam.xumax_d(fy) * d
+            esc = self.csb.ecu / xu * (xu - dc)
+            fsc = bottom_layer.rebar.fs(esc)
+            fcc = self.csb._fc_(esc) * self.conc.fd
+            asc = ast2 * fd / (fsc - fcc)
+            print("---", xu, d, esc, fsc, fcc, ast1, ast2, ast, asc)
+        return ast, asc
+
 
 """Class to repersent flanged section"""
 
@@ -297,9 +351,9 @@ class FlangedBeamSection(RectBeamSection):
         self.calc_stress_type(xu)
         k = xu / self.D
         ecy = self.csb.ecy
-        s = f"FLANGED BEAM SECTION - Web: {self.b} x {self.D}, Flange: {self.bf} x {self.Df}\n"
-        s += f"FLEXURE\nEquilibrium NA = {xu:.2f} (ec_max = {ecmax:.6f})\n"
-        s += f"Concrete: {self.conc}\n"
+        hdr0 = f"FLANGED BEAM SECTION - Web: {self.b} x {self.D}, Flange: {self.bf} x {self.Df}"
+        s = f"{header(hdr0, '~')}\n"
+        s += f"{header('FLEXURE', '=')}\nEquilibrium NA = {xu:.2f} (ec_max = {ecmax:.6f})\n\n"
         Fcw, Mcw = self.Cw(xu, ecmax)
         Fcf, Mcf = self.Cf(xu)
         Fc = Fcw + Fcf
@@ -315,7 +369,7 @@ class FlangedBeamSection(RectBeamSection):
         s += f"{Fcf/1e3:8.2f} {Mcf/1e6:8.2f}\n"
         s += f"{underline(hdr1)}\n"
         s += f"{' ':>62} {(Fcw+Fcf)/1e3:8.2f} {(Mcw+Mcf)/1e6:8.2f}\n"
-        hdr2 = f"{'fy':>6} {'Bars':>12} {'xc':>8} {'Strain':>12} {'Type':>4} {'f_sc':>8} {'f_cc':>6}"
+        hdr2 = f"{'fy':>6} {'Bars':>12} {'xc':>8} {'Strain':>12} {'Type':>4} {'f_s':>8} {'f_c':>6}"
         hdr2 += f" {'C (kN)':>8} {'M (kNm)':>8}"
         s += f"\n{hdr2}\n{underline(hdr2)}\n"
         Ft = 0.0
@@ -344,11 +398,38 @@ class FlangedBeamSection(RectBeamSection):
             s += f"{' ':>62} {C_M}\n{' ':>62} {underline(C_M, '=')}\n"
         F = 0.0 if isclose(Fcw + Fcf + Ft, 0, abs_tol=1e-10) else Fcw + Fcf + Ft
         s += f"{' ':>62} {F/1e3:8.2f} {(Mc + Mt)/1e6:8.2f}\n"
-        s += f"SHEAR\n{self.shear_steel}\n"
-        s += f"CAPACITY\n{'Mu = ':>5}{self.Mu(xu, ecmax)/1e6:.2f} kNm\n"
-        vuc, vus = self.Vu(xu)
-        vu = vuc + sum(vus)
-        s += f"{'Vu = ':>5}{vu/1e3:.2f} kN\n"
+
+        s += f"{header('SHEAR', '=')}\n"
+        tauc = self.conc.tauc(self.pt(xu))
+        area = self.b * self.eff_d(xu)
+        vuc = area * tauc
+        hdr3 = f"{'Type':>14} {' ':>14} {'tau_c':>6} {'Area (mm^2)':>16} {' ':>8} {' ':>8} {'V_uc (kN)':>8}"
+        s += f"{header(hdr3)}\n"
+        s += f"{'Concrete':>14} {' ':>14} {tauc:6.2f} {area:16.2f} {' ':>8} {' ':>8} {vuc/1e3:8.2f}\n"
+        s += f"{underline(hdr3)}\n"
+        hdr4 = f"{'Type':>14} {'Variant':>14} {'f_y':>6} {'Bars':>16} {'s_v':>8} {'A_sv':>8} {'V_us (kN)':>8}"
+        s += f"{header(hdr4)}\n"
+        vus = 0.0
+        for sh_rein in self.shear_steel.shear_reinforcement:
+            data = sh_rein.report(self.eff_d(xu))
+            s += f"{data['label']:>14} {data['type']:>14} {data['fy']:6} "
+            if data["sh_type"] in [
+                ShearRebarType.SHEAR_REBAR_VERTICAL_STIRRUP,
+                ShearRebarType.SHEAR_REBAR_INCLINED_STIRRUP,
+            ]:
+                bar_info = f"{data['legs']}-{data['bar_dia']}#"
+            else:
+                bar_info = f"{data['bars']}"
+            s += f"{bar_info:>16} {data['sv']:8.1f} {data['Asv']:8.2f} {data['Vus']/1e3:8.2f}\n"
+            vus += data["Vus"]
+        vu = f"{(vuc + vus)/1e3:8.2f}"
+        s += f"{' ':>71} {underline(vu, '=')}\n{' ':>71} {vu}\n"
+        s += (
+            f"{header('CAPACITY', '=')}\n{'Mu = ':>5}{self.Mu(xu, ecmax)/1e6:.2f} kNm\n"
+        )
+        Vuc, Vus = self.Vu(xu)
+        Vu = Vuc + sum(Vus)
+        s += f"{'Vu = ':>5}{Vu/1e3:.2f} kN\n"
         return s
 
 
@@ -422,7 +503,8 @@ class RectColumnSection:
     def report(self, xu: float) -> str:
         k = xu / self.D
         ecy = self.csb.ecy
-        s = f"RECTANGULAR COLUMN {self.b} x {self.D} xu = {xu:.2f} (k = {k:.2f})\n"
+        hdr0 = f"RECTANGULAR COLUMN {self.b} x {self.D} xu = {xu:.2f} (k = {k:.2f})"
+        s = f"{header(hdr0, '~')}\n"
         s += f"Concrete: {self.conc} Clear Cover: {self.clear_cover}\n"
         # Concrete
         fd = self.conc.fd
@@ -477,4 +559,6 @@ class RectColumnSection:
         C, M = self.C_M(xu)
         hdr3 = f"{C/1e3:8.2f} {M/1e6:8.2f}"
         s += f"{' ':>62} {underline(hdr3, '=')}\n{' ':>62} {hdr3}\n"
+        s += f"{header('CAPACITY', '=')}\n"
+        s += f"Pu = {C/1e3:10.2f} kN\nMu = {M/1e6:10.2f} kNm\n e = {M/C:10.2f} mm\n"
         return s
